function [sigm, dsbar, theta] = invar(stress)
%
% This subroutine forms the stress invariants in 2- or 3-d.
%
zero=0;
small=1e-10;
one=1;
two=2;
three=3;
six=6;
thpt5=13.5;
nst=max(size(stress));
if nst==4
    sx=stress(1);
    sy=stress(2);
    txy=stress(3);
    sz=stress(4);
    sigm=(sx+sy+sz)/three;
    dsbar=sqrt((sx-sy)*(sx-sy)+(sy-sz)*(sy-sz)+(sz-sx)*(sz-sx)+six*txy*txy)/sqrt(two);
    if dsbar<small
        theta=zero;
    else
        dx=(two*sx-sy-sz)/three;
        dy=(two*sy-sz-sx)/three;
        dz=(two*sz-sx-sy)/three;
        xj3=dx*dy*dz-dz*txy*txy;
        sine=-thpt5*xj3/(dsbar*dsbar*dsbar);
        if sine>=one
            sine=one;
        end
        if sine<-one
            sine=-one;
        end
        theta=asin(sine)/three;
    end
end
if nst==6
    sq3=sqrt(three);
    s1=stress(1);
    s2=stress(2);
    s3=stress(3);
    s4=stress(4);
    s5=stress(5);
    s6=stress(6);
    sigm=(s1+s2+s3)/three;
    d2=((s1-s2)*(s1-s2)+(s2-s3)*(s2-s3)+(s3-s1)*(s3-s1))/six+s4*s4+s5*s5+s6*s6;
    ds1=s1-sigm; 
    ds2=s2-sigm;  
    ds3=s3-sigm;
    d3=ds1*ds2*ds3-ds1*s5*s5-ds2*s6*s6-ds3*s4*s4+two*s4*s5*s6;
    dsbar=sq3*sqrt(d2);
    if dsbar<small
        theta=zero;
    else
        sine=-three*sq3*d3/(two*sqrt(d2*d2*d2));
        if sine>=one
            sine=one;
        end
        if sine<-one
            sine=-one;
        end
        theta=asin(sine)/three;
    end
end
