function [dfds, dqds, dEdp] = mcdpl(iG,phi,psi,coh,dphidE,dcohdE,dee,stress,dstrain)
%
% This subroutine forms the plastic stress/strain matrix
% for a Mohr-Coulomb material (phi,psi in radians).
%
zero=0;
pt49=0.49;
one=1;
two=2;
d3=3;
d4=4;
d6=6;
ih=max(size(stress(:,iG)));
snph=sin(phi(iG));
csph=cos(phi(iG));
snps=sin(psi(iG));
sq3=sqrt(d3);

deps(1:ih,1) = dstrain(1:ih,iG);

%niezmienniki
[sigm, dsbar, theta] = invar(stress(:,iG));
dfdphi = sigm*csph-1/3*dsbar*sin(theta)*csph + coh(iG)*snph;
dfdcoh = -csph;

A = -dfdphi*dphidE(iG) - dfdcoh*dcohdE(iG);

if ih==4
    sx=stress(1,iG);
    sy=stress(2,iG);
    txy=stress(3,iG); 
    sz=stress(4,iG); 
    t3=one/d3;
    t4=(sx+sy+sz)*t3;
    t8=sz-t4;
    t10=txy*txy;
    t16=(sx-sy)*(sx-sy);
    t18=(sy-sz)*(sy-sz);
    t20=(sz-sx)*(sz-sx);
    t25=sqrt((t16+t18+t20)/d6+t10);
    t26=t25*t25;
    t30=d3*sq3*((sx-t4)*(sy-t4)*t8-t8*t10)/two/t26/t25;
    if t30>one
        t30=one;
    end
    if t30<-one
        t30=-one;
    end
    t31=asin(t30);
    t33=sin(t31*t3);
    snth=-t33;
    if abs(snth)>pt49
        pm=-one;
        if snth<zero
            pm=one;
        end
        t2=snph/d3;
        t4=(sx-sy)*(sx-sy);
        t6=(sy-sz)*(sy-sz);
        t8=(sz-sx)*(sz-sx);
        t10=one/d6;
        t12=txy*txy;
        t14=sqrt((t4+t6+t8)*t10+t12);
        t17=one/t14/sq3;
        t18=one/two;
        t19=t17*t18;
        t21=d3+pm*snph;
        t23=two*sy;
        t24=two*sz;
        t31=two*sx;
        dfds(1,1)=t2+t19*t21*(d4*sx-t23-t24)*t10/two;
        dfds(2,1)=t2+t19*t21*(-t31+d4*sy-t24)*t10/two;
        dfds(3,1)=t17*t18*t21*txy;
        dfds(4,1)=t2+t19*t21*(-t23+d4*sz-t31)*t10/two;
        t2=snps/d3;
        t21=d3+pm*snps;
        dqds(1,1)=t2+t19*t21*(d4*sx-t23-t24)*t10/two;
        dqds(2,1)=t2+t19*t21*(-t31+d4*sy-t24)*t10/two;
        dqds(3,1)=t17*t18*t21*txy;
        dqds(4,1)=t2+t19*t21*(-t23+d4*sz-t31)*t10/two;
    else
        t1=one/d3;
        t2=snph*t1;
        t4=(sx-sy)*(sx-sy);
        t6=(sy-sz)*(sy-sz);
        t8=(sz-sx)*(sz-sx);
        t10=one/d6;
        t12=txy*txy;
        t13=(t4+t6+t8)*t10+t12;
        t14=sqrt(t13);
        t16=d3*sq3;
        t18=(sx+sy+sz)*t1;
        t19=sx-t18;
        t20=sy-t18;
        t21=t19*t20;
        t22=sz-t18;
        t25=t21*t22-t22*t12;
        t26=one/two;
        t28=t14*t14;
        t30=one/t28/t14;
        t31=t16*t25*t26*t30;
        if t31>one
            t31=one;
        end
        if t31<-one
            t31=-one;
        end
        t33=asin(t31);
        t34=t33*t1;
        t35=cos(t34);
        t36=sin(t34);
        t38=one/sq3;
        t41=one/t14*(t35+t36*snph*t38);
        t43=two*sy;
        t44=two*sz;
        t46=(d4*sx-t43-t44)*t10;
        t49=one-t1;
        t53=t19*t1*t22;
        t54=t21*t1;
        t55=t1*t12;
        t60=t16*t25;
        t61=t28*t28;
        t64=t26/t61/t14;
        t68=t16*(t49*t20*t22-t53-t54+t55)*t26*t30-d3/two*t60*t64*t46;
        t70=d3*d3;
        t71=sq3*sq3;
        t73=t25*t25;
        t74=two*two;
        t77=t13*t13;
        t83=sqrt(one-t70*t71*t73/t74/t77/t13);
        t84=one/t83;
        t85=t84*t1;
        t89=t2*t38;
        t94=two*sx;
        t97=(-t94+d4*sy-t44)*t10;
        t101=t1*t20*t22;
        t111=t16*(-t101+t19*t49*t22-t54+t55)*t26*t30-d3/two*t60*t64*t97;
        t129=-two*t16*t22*txy*t26*t30-d3*t60*t64*txy;
        t140=(-t43+d4*sz-t94)*t10;
        t152=t16*(-t101-t53+t21*t49-t49*t12)*t26*t30-d3/two*t60*t64*t140;
        dfds(1,1)=t2+t41*t46/two+t14*(-t36*t68*t85+t35*t68*t84*t89);
        dfds(2,1)=t2+t41*t97/two+t14*(-t36*t111*t85+t35*t111*t84*t89);
        dfds(3,1)=t41*txy+t14*(-t36*t129*t85+t35*t129*t84*t89);
        dfds(4,1)=t2+t41*t140/two+t14*(-t36*t152*t85+t35*t152*t84*t89);
        t2=snps*t1;
        t41=one/t14*(t35+t36*snps*t38);
        t89=t2*t38;
        dqds(1,1)=t2+t41*t46/two+t14*(-t36*t68*t85+t35*t68*t84*t89);
        dqds(2,1)=t2+t41*t97/two+t14*(-t36*t111*t85+t35*t111*t84*t89);
        dqds(3,1)=t41*txy+t14*(-t36*t129*t85+t35*t129*t84*t89);
        dqds(4,1)=t2+t41*t140/two+t14*(-t36*t152*t85+t35*t152*t84*t89);
    end
end
if ih==6
    sx=stress(1,iG);
    sy=stress(2,iG);
    sz=stress(3,iG);  
    txy=stress(4,iG); 
    tyz=stress(5,iG); 
    tzx=stress(6,iG); 
    t3=one/d3;
    t4=(sx+sy+sz)*t3;
    t5=sx-t4;
    t6=sy-t4;
    t8=sz-t4;
    t10=tyz*tyz;
    t12=tzx*tzx;
    t14=txy*txy;
    t23=(sx-sy)*(sx-sy);
    t25=(sy-sz)*(sy-sz);
    t27=(sz-sx)*(sz-sx);
    t32=sqrt((t23+t25+t27)/d6+t14+t10+t12);
    t33=t32*t32;
    t37=d3*sq3*(t5*t6*t8-t5*t10-t6*t12-t8*t14+two*txy*tyz*tzx)/two/t33/t32;
    if t37>one
        t37=one;
    end
    if t37<-one
        t37=-one;
    end
    t38=asin(t37);
    t40=sin(t38*t3);
    snth=-t40;
    if abs(snth)>pt49
        pm=-one;
        if snth<zero
            pm=one;
        end
        t2=snph/d3;
        t4=(sx-sy)*(sx-sy);
        t6=(sy-sz)*(sy-sz);
        t8=(sz-sx)*(sz-sx);
        t10=one/d6;
        t12=txy*txy;
        t13=tyz*tyz;
        t14=tzx*tzx;
        t16=sqrt((t4+t6+t8)*t10+t12+t13+t14);
        t19=one/t16/sq3;
        t20=one/two;
        t21=t19*t20;
        t23=d3+pm*snph;
        t25=two*sy;
        t26=two*sz;
        t33=two*sx;
        t48=t20*t23;
        dfds(1,1)=t2+t21*t23*(d4*sx-t25-t26)*t10/two;
        dfds(2,1)=t2+t21*t23*(-t33+d4*sy-t26)*t10/two;
        dfds(3,1)=t2+t21*t23*(-t25+d4*sz-t33)*t10/two;
        dfds(4,1)=t19*t48*txy;
        dfds(5,1)=t19*t48*tyz;
        dfds(6,1)=t19*t48*tzx;
        t2=snps/d3;
        t23=d3+pm*snps;
        t48=t20*t23;
        dqds(1,1)=t2+t21*t23*(d4*sx-t25-t26)*t10/two;
        dqds(2,1)=t2+t21*t23*(-t33+d4*sy-t26)*t10/two;
        dqds(3,1)=t2+t21*t23*(-t25+d4*sz-t33)*t10/two;
        dqds(4,1)=t19*t48*txy;
        dqds(5,1)=t19*t48*tyz;
        dqds(6,1)=t19*t48*tzx;
    else
        t1=one/d3;
        t2=snph*t1;
        t4=(sx-sy)*(sx-sy);
        t6=(sy-sz)*(sy-sz);
        t8=(sz-sx)*(sz-sx);
        t10=one/d6;
        t12=txy*txy;
        t13=tyz*tyz;
        t14=tzx*tzx;
        t15=(t4+t6+t8)*t10+t12+t13+t14;
        t16=sqrt(t15);
        t18=d3*sq3;
        t20=(sx+sy+sz)*t1;
        t21=sx-t20;
        t22=sy-t20;
        t23=t21*t22;
        t24=sz-t20;
        t29=two*txy;
        t32=t23*t24-t21*t13-t22*t14-t24*t12+t29*tyz*tzx;
        t33=one/two;
        t35=t16*t16;
        t37=one/t35/t16;
        t39=t18*t32*t33*t37;
        if t39>one
            t39=one;
        end
        if t39<-one
            t39=-one;
        end
        t40=asin(t39);
        t41=t40*t1;
        t42=cos(t41);
        t43=sin(t41);
        t45=one/sq3;
        t48=one/t16*(t42+t43*snph*t45);
        t50=two*sy;
        t51=two*sz;
        t53=(d4*sx-t50-t51)*t10;
        t56=one-t1;
        t60=t21*t1*t24;
        t61=t23*t1;
        t63=t1*t14;
        t64=t1*t12;
        t69=t18*t32;
        t70=t35*t35;
        t73=t33/t70/t16;
        t77=t18*(t56*t22*t24-t60-t61-t56*t13+t63+t64)*t33*t37-d3/two*t69*t73*t53;
        t79=d3*d3;
        t80=sq3*sq3;
        t82=t32*t32;
        t83=two*two;
        t86=t15*t15;
        t92=sqrt(one-t79*t80*t82/t83/t86/t15);
        t93=one/t92;
        t94=t93*t1;
        t98=t2*t45;
        t103=two*sx;
        t106=(-t103+d4*sy-t51)*t10;
        t110=t1*t22*t24;
        t113=t1*t13;
        t122=t18*(-t110+t21*t56*t24-t61+t113-t56*t14+t64)*t33*t37-d3/two*t69*t73*t106;
        t133=(-t50+d4*sz-t103)*t10;
        t145=t18*(-t110-t60+t23*t56+t113+t63-t56*t12)*t33*t37-d3/two*t69*t73*t133;
        t166=t18*(-two*t24*txy+two*tyz*tzx)*t33*t37-d3*t69*t73*txy;
        t186=t18*(-two*t21*tyz+t29*tzx)*t33*t37-d3*t69*t73*tyz;
        t206=t18*(-two*t22*tzx+t29*tyz)*t33*t37-d3*t69*t73*tzx;
        dfds(1,1)=t2+t48*t53/two+t16*(-t43*t77*t94+t42*t77*t93*t98);
        dfds(2,1)=t2+t48*t106/two+t16*(-t43*t122*t94+t42*t122*t93*t98);
        dfds(3,1)=t2+t48*t133/two+t16*(-t43*t145*t94+t42*t145*t93*t98);
        dfds(4,1)=t48*txy+t16*(-t43*t166*t94+t42*t166*t93*t98);
        dfds(5,1)=t48*tyz+t16*(-t43*t186*t94+t42*t186*t93*t98);
        dfds(6,1)=t48*tzx+t16*(-t43*t206*t94+t42*t206*t93*t98);
        t2=snps*t1;
        t48=one/t16*(t42+t43*snps*t45);
        t98=t2*t45;
        dqds(1,1)=t2+t48*t53/two+t16*(-t43*t77*t94+t42*t77*t93*t98);
        dqds(2,1)=t2+t48*t106/two+t16*(-t43*t122*t94+t42*t122*t93*t98);
        dqds(3,1)=t2+t48*t133/two+t16*(-t43*t145*t94+t42*t145*t93*t98);
        dqds(4,1)=t48*txy+t16*(-t43*t166*t94+t42*t166*t93*t98);
        dqds(5,1)=t48*tyz+t16*(-t43*t186*t94+t42*t186*t93*t98);
        dqds(6,1)=t48*tzx+t16*(-t43*t206*t94+t42*t206*t93*t98);
    end
end
dEdp = (dfds' * dee * deps)./(dfds' * dee * dqds + A);  
