function [Disp, Stress, EDP] = shearQ8( MaterialConf, CalcConf, MeshConf, opt )       
    % *** OBLICZENIA ***
    
    %            4        
    % 3 o--------o--------o 5
    %   |                 |
    %   |                 |
    % 2 o                 o 6
    %   |                 |
    %   |                 |
    % 1 o--------o--------o 7
    %            8
    %
    %    1 u1x = 0
    %    2 u1y = 0
    %    3 u2x = d/2
    %    4 u2y = ?
    %    5 u3x = d
    %    6 u3y = ?
    %    7 u4x = d
    %    8 u4y = ?
    %    9 u5x = d
    %   10 u5y = ?
    %   11 u6x = d/2
    %   12 u6y = ?
    %   13 u7x = 0
    %   14 u7y = 0
    %   15 u8x = ?
    %   16 u8y = 0
    
    disp('Prosze czekac...');
    
    MAT = MaterialConf;
    CAL = CalcConf;
    MSH = MeshConf;
    
    %przeliczenie jednostek
    MAT.Fi = MAT.Fi .* pi ./ 180;
    MAT.Psi = MAT.Psi .* pi ./ 180;
    
    %poszerzenie zakresow do interpolacji
    tmp = [MAT.Fi(1,1) MAT.Fi(1,2) MAT.Fi(1,2) MAT.Fi(1,3) MAT.Fi(1,3)];
    MAT.Fi = zeros(1,5);
    MAT.Fi = tmp;
    tmp = [MAT.c(1,1) MAT.c(1,2) MAT.c(1,2) MAT.c(1,3) MAT.c(1,3)];
    MAT.c = zeros(1,5);
    MAT.c = tmp;
    tmp = [MAT.Psi(1,1) MAT.Psi(1,2) MAT.Psi(1,2) MAT.Psi(1,3) MAT.Psi(1,3)];
    MAT.Psi = zeros(1,5);
    MAT.Psi = tmp;
    tmp = [ 0 MAT.Edpf(1,1) MAT.Edpf(1,2) MAT.Edpf(1,3) 100];
    MAT.Edpf = zeros(1,5);
    MAT.Edpf = tmp;
    tmp = [ 0 MAT.Edpc(1,1) MAT.Edpc(1,2) MAT.Edpc(1,3) 100];
    MAT.Edpc = zeros(1,5);
    MAT.Edpc = tmp;    
    tmp = [ 0 MAT.Edpp(1,1) MAT.Edpp(1,2) MAT.Edpp(1,3) 100];
    MAT.Edpp = zeros(1,5);
    MAT.Edpp = tmp;

    %przygotowanie wynikow
    Disp = zeros(1,CAL.Incr+1);
    Stress = zeros(4,CAL.Incr+1);
    EDP = zeros(1,CAL.Incr+1);
    
    %porzadkowanie wspolrzednych wezlow
    for j=1:8
        Pnts(j,1) = MSH.Nodes( MSH.Elem.nod(j), 1 );
        Pnts(j,2) = MSH.Nodes( MSH.Elem.nod(j), 2 );
    end
    
    %zerowanie wynikow
    STRESS = zeros(4,9);
    STRAIN = zeros(4,9);
    DEVIATORIC_PLASTIC_STRAIN = zeros(1,9);
    Disp(1,1) = 0;
    EDP(1,1) = 0;
            
    %stan K0 po konsolidacji
    STRESS = [CAL.K0 CAL.K0 CAL.K0 CAL.K0 CAL.K0 CAL.K0 CAL.K0 CAL.K0 CAL.K0; 1 1 1 1 1 1 1 1 1; 0 0 0 0 0 0 0 0 0; CAL.K0 CAL.K0 CAL.K0 CAL.K0 CAL.K0 CAL.K0 CAL.K0 CAL.K0 CAL.K0];
    STRESS = (CAL.NormPres - CAL.PorePres) .* STRESS;
    Stress(1:4,1) = STRESS(1:4,9);

    %macierz sprezystosci
    De = deemat(MAT.E,MAT.ni,4);
    Du = [MAT.Ke MAT.Ke 0 MAT.Ke; MAT.Ke MAT.Ke 0 MAT.Ke; 0 0 0 0; MAT.Ke MAT.Ke 0 MAT.Ke];
    D = De + Du;
    
    %macierz sztywnosci
    K = zeros(16,16);
    for iG = 1:9        
        [B, detJ] = BQ8_2D( iG, Pnts, MSH );
        BDB = B' * D * B;
        GP = GaussQ8( iG );
        k = GP.w .* detJ .* BDB;        
        K = K + k;
    end    

    %zadane niezerowe przemieszczenia
    disps = [MSH.Disps./2; MSH.Disps; MSH.Disps; MSH.Disps; MSH.Disps./2];   

    %macierz sztywnosci po uwzglednieniu podpor
    Kc = zeros(6,6);
    Kc = [K(4:2:12,4:2:12) K(4:2:12,15); K(15,4:2:12) K(15,15)]; 
    
    %elementy macierzy sztywnosci mnozone przez zadane niezerowe przemieszczenia
    kf = zeros(6,5);
    kf = [K(4:2:12,3:2:11); K(15,3:2:11)];
        
    %wektor przyrostu obciazen z zadanych niezerowych przyrostow przemieszczen
    dfp = kf * disps;
    
    %przyrost obciazen przerzucony na praw stron - do sil
    dfp = -1 .* dfp;
                               
    %odwrocenie macierzy sztywnosci
    iKc = inv(Kc);
    
    u = zeros(6,1);
    U = zeros(16,1);
                
    %petla uwzgledniania kolejnych przyrostow obciazen dfp
    for inc = 1:CAL.Incr
        %zerowania
        iter = 0;  
        dfr = zeros(6,1);
        dFr = zeros(16,1);
        depsvp = zeros(4,9); 
        
        %aktualny stan naprezenia
        STRESS(2,1:9) = CAL.NormPres - CAL.PorePres;
        sig0 = STRESS;
        
        %skumulowany dewiator odksztalcen plastycznych
        Edp0 = DEVIATORIC_PLASTIC_STRAIN;
        dEdp = zeros(1,9);

        %wartosci parametrow wytrzymalosciowych i ich pochodnych
        [Fi coh Psi dFidE dcohdE] = updateMAT( MAT, Edp0 );                    
        
        %petla iteracji lepkoplastycznych        
        ctrl = 1;
        while ctrl > 0        
            
            %licznik
            iter = iter + 1;
            
            %uaktualnienie obciazen: zadanego i redystrybujacego
            df = dfp + dfr;
        
            %rozwiazanie ukladu rownan
            du = iKc * df;
            
            %norma aktualnego wektora przemieszczen
            ndu_next = norm(du);
    
            %uzupelnienie przyrostu przemieszczen o zadane wartosci
            dU = [0; 0; MSH.Disps./2; du(1); MSH.Disps; du(2); MSH.Disps; du(3); MSH.Disps; du(4); MSH.Disps./2; du(5); 0; 0; du(6); 0];            
                
            %stan odksztalcenia i naprezenia
            for iG = 1:9
                %macierz B, jakobian i rzedna x pkt. Gaussa
                [B detJ] = BQ8_2D( iG, Pnts, MSH );
                
                %przyrost odksztalcen calkowitych
                deps(:,iG) = B * dU;
        
                %przyrost naprezen od sprezystej czesci przyrostu odksztalcen
                dsig(:,iG) = De * ( deps(:,iG) - depsvp(:,iG) );
        
                %uaktualniony stan naprezenia
                dsig(2,iG) = 0;
                sig(:,iG) = sig0(:,iG) + dsig(:,iG);
                                
                %kryterium plastycznosci
                Fc = Fmc( sig(:,iG), Fi(iG), coh(iG) );
            
                %sprawdzenie kryterium, redystrybucja stanu naprezenia
                if Fc > 0
                    %odksztalcenia lepkoplastyczne
                                        
                    %pochodna kryterium i potencjalu plastycznego
                    [dFds dPds dEdp(iG)] = mcdpl(iG,Fi,Psi,coh,dFidE,dcohdE,D,sig,deps);
                    
                    %predkosc odksztalcen lepkoplastycznych
                    depsvpt = Fc .* dPds;
                
                    %maksymalny krok czasowy
                    DT = 4.*(1 + MAT.ni).*(1 - 2.*MAT.ni)./(MAT.E.*(1 - 2.*MAT.ni + sin(Fi(iG)).^2));
                
                    %kumulowany przyrost odksztalcen lepkoplastycznych
                    depsvp(:,iG) = depsvp(:,iG) + DT .* depsvpt;
              
                    %przyrost naprezen lepkoplastycznych
                    dsigvp = De * ( DT .* depsvpt );
                
                    %kumulowany przyrost obciazen korygujacych stan naprezenia
                    GP = GaussQ8( iG );
                    BDdevp = B' * dsigvp;
                    dFr = dFr + detJ .* GP.w .* BDdevp;
                end
            end
            %pierwszy warunek zatrzymania
            if iter == CAL.IterMax
                ctrl = 0;
            end
     
            %drugi warunek zatrzymania
            if iter > 1
                nd = (ndu_next - ndu_prev)./ndu_prev;
                if nd < CAL.IterTol
                    ctrl = 0;
                end
            end
            ndu_prev = ndu_next;
                
            %przygotowanie wektora obciazen korygujacych
            dfr = [dFr(4:2:12); dFr(15)];
        end
                
        %uaktualnienie wynikow
        STRESS = STRESS + dsig;
        STRAIN = STRAIN + deps;
        DEVIATORIC_PLASTIC_STRAIN = DEVIATORIC_PLASTIC_STRAIN + dEdp;
        u = u + du;
        U = U + dU;
               
        %wyniki zwracane przez funkcje (dla wewnetrznego punktu Gaussa)
        Disp(1,inc+1) = MSH.Disps.*inc;
        Stress(1:4,inc+1) = STRESS(1:4,9);
        EDP(1,inc+1) = DEVIATORIC_PLASTIC_STRAIN(1,9);
    end   
    
if opt > 0
    x = [MSH.Nodes(1:8,1); MSH.Nodes(1,1)];
    y = [MSH.Nodes(1:8,2); MSH.Nodes(1,2)];
    dx = [U(1:2:15); U(1)];
    dy = [U(2:2:16); U(2)];
    xd = x + dx .* opt;
    yd = y + dy .* opt;
    plot(x,y,'o-',xd,yd,'*--');
    disp('Nacisnij dowolny klawisz');
    pause
end

disp('OK');